local ShowMyDPS = LibStub("AceAddon-3.0"):NewAddon("ShowMyDPS", "AceConsole-3.0", "AceEvent-3.0")

DBDefault = {
	char = {
		handleDPS = true,
		isLock = false,
		isShown = true,
	}
}

function ShowMyDPS:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SMDdb", DBDefault)
	self:RegisterChatCommand("smd","SlashHandler")
end

function ShowMyDPS:OnEnable()
	self:CreateFrame()
	self:RegisterEvent("PLAYER_REGEN_ENABLED","LeaveCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED","EnterCombat")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED","AddAmount")
end

function ShowMyDPS:CreateFrame()

	smdFrame = CreateFrame("Frame", "ShowMyDPSFrame", UIParent, "GameTooltipTemplate")
	smdFrame:SetPoint("CENTER", UIParent, self.db.char.x and "BOTTOMLEFT" or "CENTER", self.db.char.x or 0, self.db.char.y or 0)
	smdFrame:EnableMouse(true)
	smdFrame:SetToplevel(true)
	smdFrame:SetMovable(true)
	smdFrame:SetFrameStrata("LOW")
	smdFrame:SetWidth(220)
	smdFrame:SetHeight(25)
	if self.db.char.isShown then
		smdFrame:Show()
	end

	local backdrop_header = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile=1, tileSize=16, edgeSize = 16,
			insets = {left = 5, right = 5, top = 5, bottom = 5}}

	smdFrame:SetBackdrop(backdrop_header);
	smdFrame:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
	smdFrame:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)

	smdFrame.text = smdFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall")
	smdFrame.text:SetPoint("CENTER", smdFrame, "CENTER", 0, 0)
	smdFrame.text:Show()

	smdFrame:SetScript("OnMouseDown", function(frame, button) 
		if not self.db.char.isLock and button ~= "RightButton" then
			frame.isMoving = true
			frame:StartMoving()	
		end
	end)

	smdFrame:SetScript("OnMouseUp", function(frame, button) 
		if not self.db.char.isLock and button ~= "RightButton" then
			if( frame.isMoving ) then
				frame.isMoving = nil
				frame:StopMovingOrSizing()
				self.db.char.x, self.db.char.y = frame:GetCenter()
			end
		else
			if self.db.char.handleDPS then self.db.char.handleDPS = false else self.db.char.handleDPS = true end
			self:AddAmount()
		end
	end)
	
	ShowMyDPS.Frame = smdFrame
end

function ShowMyDPS:SlashHandler(msg)
	if msg == "lock" then
		if self.db.char.isLock then self.db.char.isLock = false else self.db.char.isLock = true end
	elseif msg == "show" then
		if ShowMyDPS.Frame:IsShown() then ShowMyDPS.Frame:Hide() self.db.char.isShown = false else ShowMyDPS.Frame:Show() self.db.char.isShown = true end
	elseif not msg or msg == "" then
		self:Printf("/smd lock : toggle lock/unlock")
		self:Printf("/smd show : toggle show/hide")
	end
end

local cbtDuration, cbtStart
local cbtDPS
local cbtDmgAmount = 0
local cbtHPS
local cbtHealAmount = 0

function ShowMyDPS:EnterCombat()
	cbtDmgAmount = 0
	cbtHealAmount = 0
	cbtStart = GetTime() - 3.5
end

function ShowMyDPS:AddAmount(...)
	if not arg5 or bit.band(arg5, 0x1111) == 0x1111 or bit.band(arg5, 0x511) == 0x511 then -- Checking source
	
		-- DPS crap
		if (arg2 and string.find(arg2, "_DAMAGE")) then
			if not arg12 then -- Checking dps type
				cbtDmgAmount = cbtDmgAmount + arg9 -- Melee swing
			else
				cbtDmgAmount = cbtDmgAmount + arg12 -- Anything else
			end
		end
		
		-- Heal crap
		if (arg2 and string.find(arg2, "_HEAL")) then
			cbtHealAmount = cbtHealAmount + ( arg12 - arg13 ) -- Only effective healing
		end
		
	
		-- Get recount-like feature
		cbtStart = cbtStart or GetTime() - 3.5

		-- Calculation
		cbtDuration = GetTime() - cbtStart
		cbtDuration = (floor((cbtDuration * 100) + 0.5))/100
		if cbtDmgAmount then cbtDPS = (floor((cbtDmgAmount/cbtDuration * 100) + 0.5))/100 end
		if cbtHealAmount then cbtHPS = (floor((cbtHealAmount/cbtDuration * 100) + 0.5))/100 end
		
		
		-- Adapt
		if cbtDuration > 60 then cbtDuration = floor(cbtDuration/60).." min "..mod(cbtDuration,60) end
		if cbtDmgAmount and cbtDmgAmount > 1000000 then cbtDmgAmount = floor(cbtDmgAmount/1000).."k" end
		if cbtHealAmount and cbtHealAmount > 1000000 then cbtHealAmount = floor(cbtHealAmount/1000).."k" end
		
		-- Show
		if self.db.char.handleDPS then
			if cbtDmgAmount and cbtDmgAmount > 0 then
				ShowMyDPSFrameText:SetText(format("%s | %s sec | %s dps",cbtDmgAmount,cbtDuration,cbtDPS))
			else
				ShowMyDPSFrameText:SetText("")
			end
		else
			if cbtHealAmount and cbtHealAmount > 0 then
				ShowMyDPSFrameText:SetText(format("%s | %s sec | %s hps",cbtHealAmount,cbtDuration,cbtHPS))
			else
				ShowMyDPSFrameText:SetText("")
			end
		end
		
	end
end

function ShowMyDPS:LeaveCombat()
	cbtDmgAmount = 0
	cbtHealAmount = 0
	cbtStart = nil
end